<?php

	/**
	 * Dieser Service stellt eine Server2Server-Schnittstelle (API), zur Buchung von Transaktionen ber die Zahlungsart Kreditkarte, zur Verfgung (Creditcard - API.Event).
	 * 
	 *  Voraussetzungen:
	 *  1. Ein Account bei micropayment - Anmeldung unter www.micropayment.de
	 *  2. Ein Projekt anlegen, damit der Testmodus zur Verfgung steht
	 *  3. Anlegen der Benachrichtigungs-URL und Serverzugriffsberechtigungen im ControlCenter
	 *  4. Eine Vertrags-Unternehmen Nummer (VU-Nummer), diese kann im ControlCenter beantragt werden.
	 * 
	 *  Integration
	 *  Um Kreditkartenzahlungen sicher und PCI:DSS konform verarbeiten zu knnen, gibt es folgende Arten der Integration zur Abwicklung von einmaligen und wiederkehrenden Zahlungen:
	 * 
	 *  1. Hybrid-Modus/Web2API https://sipg.micropayment.de/public/creditcard/WEB2API/?language=de
	 *  Sie erhalten fr Ihr Projekt ein fertiges Bezahlfenster in dem die initiale Eingabe der Kreditkarteninformationen stattfindet. Sie kommen mit den Daten nicht in Berhrung. Nach der erfolgreichen Zahlung bermitteln wir an Ihr System eine CustomerId. Mit dieser customerId knnen Sie dann per API Folgetransaktionen durchfhren.
	 * 
	 *  2. Tokenisation und Iframelsung https://techdoc.micropayment.de/payment/creditcard_api/token.pdf
	 *  Sichere Steuerung von Kreditkartendaten und Gestaltung eines eigenen Bezahlfensters im Look & Feel des Webprojektes.
	 * 
	 *  3. Die voll API Integration mit direktem Kontakt zu Kreditkartendaten
	 *  ACHTUNG: ist nur zulssig, wenn Sie den richtigen PCI:DSS Status haben und berechtigt sind Kreditkartendaten zu verarbeiten / speichern.
	 * 
	 *  4. Fr Zahlungen die 3D Secure version 2.x untersttzen sollen, nutzen Sie bitte die Funktionen `transactionPurchase` oder `tokenPurchase` bzw.
	 *  `transactionAuthorization` oder `tokenAuthorization` und setzen start3d=1. Die Funktion `check3dAvailability` ist nicht fr Version 2.x ausgelegt!
	 * 
	 *  Konfiguration:
	 *  Loggen Sie sich in das ControlCenter Ihres Accounts ein und begeben sich zu dem Menpunkt "Meine Konfiguration":
	 *  - legen Sie ein Projekt an. Konfiguration->Projekte
	 *  - unter Konfiguration->Accesskey finden Sie Ihren "AccessKey" den Sie fr die Verwendung der API bentigen
	 *  - Unter "Zahlarten" konfigurieren und aktivieren Sie Creditcard - API.Event
	 *  - im Untermenpunkt "Zugriffsberechtigungen" konfigurieren Sie bitte die IP-Adresse/IP-Range der Server, von dem aus Sie auf das API zugreifen mchten
	 *  - Unter "Zahlarten" konfigurieren und aktivieren Sie Creditcard ? Event fr die initiale Transaktion (siehe Integration 1 o. 2)
	 * 
	 *  Hinweise zum TestModus:
	 *  - Aktivierung der Testumgebung erfolgt in der jeweiligen Funktion ber den Parameter `testMode`
	 *  - fr den TestModus stehen folgende Test-Kartennummern zur Verfgung:
	 * 	- VISA: 4111111111111111
	 * 	- MASTER: 5454545454545454
	 * 	- AMEX: 343434343434343
	 *  - um erfolgreiche Buchungen durchzufhren, ist als CVC2-Code die 666 zu verwenden, bei allen Anderen gilt die Transaktion als fehlgeschlagen
	 *  - um einen spezifischen Terminal-Fehlercode auszulsen, ist als CVC2-Code die 555 zu verwenden:
	 *    Der als Integerwert (bspw. 534) angefordertete Endkundenbetrag liefert im Eurocent-Anteil (bspw. 34) den Fehlercode (`transactionResultCode`) zurck - z.B. 534 => ipg34.
	 *  	 Bitte beachten Sie, dass das erwartete Ergebnis nur bei der Verwendung der Whrung (`currency`) "EUR" erreicht wird.
	 *    Achtung: bei 3DSecure kann dieses Fehlerverhalten nur mit der Karte VISA: 4747474747474747 erreicht werden.
	 * 
	 *  bermittlung von Adressdaten:
	 *  Sollten Ihnen die Adressdaten Ihrer Endkunden zur Verfgung stehen, empfehlen wir diese mittles "addressSet" im Kundendatensatz zu hinterlegen.
	 *  Die im Kundendatensatz (customerCreate, customerSet) hinterlegten Adressdaten, werden zum Einen fr das Adress-Verifikations-System (AVS, verfgbar ab Version 1.5) bentigt,
	 *  zum Anderen werden diese Daten bei Chargeback-Anfragen (s.g. Beleganforderungen von Issuer und Acquirerer) verwendet.
	 * 
	 *  Fehlercodes:
	 *  Eine bersicht ber die mglichen Fehlercodes des API als auch des Terminals finden Sie unter folgender Adresse:
	 *  https://sipg.micropayment.de/public/creditcard/ERRORCODES/
	 * 
	 *  Updates/Changelog:
	 *  Wenn Sie bisher eine ltere Version dieses APIs verwenden, knnten Sie sich im Changelog einen berblick ber die Neuerungen verschaffen:
	 *  https://sipg.micropayment.de/public/creditcard/CHANGELOG/
	 * 
	 *  Hinweis zu Hoch-Traffic-Anwendungen:
	 *  Grundstzlich kann ein Kreditkartenterminal immer nur eine Buchung zur selben Zeit ausfhren.
	 *  Bei normalem Traffic fngt eine Warteschlange gleichzeitige Anfragen auf.
	 *  Wenn aber parallele Transaktionen erforderlich sind (z.B. bei Trafficspitzen), knnen Sie zur Lastverteilung bei unserem Support zustzliche Terminals ordern.
	 *
	 * @copyright 2020 micropayment GmbH
	 * @link http://www.micropayment.de/
	 * @author Yves Berkholz, Guido Franke
	 * @version 2.3
	 * @created 2020-01-22 15:49:09
	 */
	interface IMcpCreditcardService_v2_3 {

		/**
		 * Prüft ob die Kreditkarte am 3D-Secure Verfahren teilnimmt. Die Teilnahme wird durch die Bereitstellung einer URL bestätigt.
		 * 
		 *  Achtung, 3D Secure version 2.0 wird nicht unterstüzt!
		 * 
		 *  Die Authentifizierung kann nur durchgeführt werden, wenn sowohl der Issuer als auch der Karteninhaber für
		 *  3-D Secure angemeldet sind. In diesem Fall wird die Kreditkarte als "enrolled" (deutsch: "eingetragen") bezeichnet.
		 * 
		 *  Wenn die Kreditkarte nicht "enrolled" ist, kann mit dem herkömmlichen Payment-Prozess weiterverfahren werden, status: BILLING.
		 * 
		 *  Wenn die Karte am 3-D Secure-Verfahren teilnimmt, muss der Kunde mit seinem Browser auf den Access Control Server
		 *  der Issuer Bank umgeleitet werden, status: START3D. Der Access Control Server nimmt die eigentliche Authentifizierung vor.
		 *  Dafür muss das Frontend-System (z. B. Shop) den Kunden-Browser an die angegebene 'forwardUrl' weiterleiten.
		 * 
		 *  Nach Abschluß des Verfahrens wird der Kunde an die angegeben 'returnUrl' weitergeleitet. An die Adresse wird außerdem der
		 *  Parameter 'check3dId' angehängt. Dieser wird für die Funktion 'verify3dNotification' zur Gegenprobe benötigt.
		 * 
		 *  Bei status: DECLINED ist die Kreditkarte nicht "enrolled", aber für den Merchant / account wurde das als Pflicht definiert.
		 *  Bei status: FAILURE ist ein Verarbeitungsfehler aufgetreten.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * @param string $returnUrl Url an die nach Abschluß des 3D-Secure Verfahrens weitergeleitet werden soll
		 * @param string $cvc2 (default=null)  CVC2-Code der Kreditkarte
		 * 
		 * @return struct 
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $secureStatus BILLING | START3D | DECLINED | FAILURE
		 * @result string $forwardUrl (default=null)  Wenn gesetzt, kann der Kunde an diese Adresse weitergeleitet werden
		 */
		public function check3dAvailability($accessKey, $testMode=0, $sessionId, $returnUrl, $cvc2=null);

		/**
		 * Prüft das Ergebnis der 3DSecure Funktionen.
		 * 
		 *  Bei den 3DSecure Funktionen die den `returnUrl` erwarten, wird hiermit die Gegenprüfung gemacht.
		 *  Nach Abschluß des 3DSecure-Verfahrens wird der Kunde an diese `returnUrl` weitergeleitet.
		 *  An die angegebene 'returnUrl' wird der Parameter check3dId angehängt und dient zur Abfrage der Transaktionsdaten.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $check3dId entsprechender Wert von check3dAvailability
		 * 
		 * @return struct 
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $sessionId ID des Vorgangs
		 * @result string $transactionId Transaktionsnummer
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 * @result TransactionType $type Art der Transaktion
		 */
		public function verify3dNotification($accessKey, $testMode=0, $check3dId);

		/**
		 * Führt eine Transaktion zur sofortigen Buchung, des durch `sessionCreate` definierten Betrages, durch.
		 * 
		 *  Zur Durchführung der Transaktion ist es notwendig das Sie mittels `sessionCreate`  bzw. `tokenSessionCreate` einen Bezahlvorgang erstellt haben.
		 * 
		 *  Beim $secureStatus == START3D wird das 3D-Secure Verfahren mit Weiterleitung auf $forwardUrl gestartet. In diesem
		 *  Fall erfahren Sie das Ende der Zahlung erst beim Aufruf der $returnUrl. An $returnUrl wird außerdem der Parameter 'check3dId' angehängt.
		 *  Dieser wird für die Funktion 'verify3dNotification' zur Gegenprobe benötigt.
		 * 
		 *  Beim $secureStatus == BILLING sowie bei fehlerhaften Transaktionen erhalten Sie sofort das Ergebnis der Transaktion.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * @param string $cvc2 (default=null)  CVC2-Code der Kreditkarte, muß min einmal pro Kreditkarte/Verfallszeit angegeben worden sein
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * @param integer $avsCheck (default=0)  de/aktiviert das AddressVerificationSystem
		 * @param integer $recurring (default=0)  Wenn 1 kann der Vorgang als Teil einer Zahlungsfolge markiert werden.
		 * @param integer $start3d (default=0)  Wenn 1 wird der 3dSecure Vorgang gestartet. Entsprechender Response muss beachtet werden.
		 * @param string $returnUrl (default=null)   Url an die nach Abschluß des 3D-Secure Verfahrens weitergeleitet werden soll.
		 * 
		 * @return struct 
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 * @result AVSResult $avsResult (default=null)  wenn ein AVS-Check angefordert wurde dessen Ergebnis
		 * @result string $secureStatus BILLING | START3D | DECLINED | FAILURE
		 * @result string $forwardUrl (default=null)  Wenn gesetzt, muß der Kunde an diese Adresse weitergeleitet werden um daß 3D-Secure Verfahren zu durchlaufen.
		 */
		public function transactionPurchase($accessKey, $testMode=0, $sessionId, $cvc2=null, $fraudDetection=1, $avsCheck=0, $recurring=0, $start3d=0, $returnUrl=null);

		/**
		 * Führt eine Transaktion zur Vorautorisierung, des durch `sessionCreate` definierten Betrages, durch.
		 * 
		 *  Der gewünschten Betrag wird für Sie reserviert bis:
		 *  - Sie den Betrag mittels `transactionReversal` wieder freigeben
		 *  - Sie den Gesamt- oder Teilbetrag mit `transactionCapture` buchen
		 *  - die Reservierung nach ca. 14 Tagen automatisch wieder freigegeben wird
		 * 
		 *  Zur Durchführung der Transaktion ist es notwendig das Sie mittels `sessionCreate`  bzw. `tokenSessionCreate` einen Bezahlvorgang erstellt haben.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * @param string $cvc2 (default=null)  CVC2-Code der Kreditkarte, muß min einmal pro Kreditkarte/Verfallszeit angegeben worden sein
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * @param integer $avsCheck (default=0)  de/aktiviert das AddressVerificationSystem
		 * @param integer $start3d (default=0)  Wenn 1 wird der 3dSecure Vorgang gestartet. Entsprechender Response muss beachtet werden.
		 * @param string $returnUrl (default=null)   Url an die nach Abschluß des 3D-Secure Verfahrens weitergeleitet werden soll.
		 * 
		 * @return struct 
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 * @result AVSResult $avsResult (default=null)  wenn ein AVS-Check angefordert wurde dessen Ergebnis
		 * @result string $secureStatus BILLING | START3D | DECLINED | FAILURE
		 * @result string $forwardUrl (default=null)  Wenn gesetzt, muß der Kunde an diese Adresse weitergeleitet werden um daß 3D-Secure Verfahren zu durchlaufen.
		 */
		public function transactionAuthorization($accessKey, $testMode=0, $sessionId, $cvc2=null, $fraudDetection=1, $avsCheck=0, $start3d=0, $returnUrl=null);

		/**
		 * sofortige Buchung
		 *  Kombiniert die Funktionen 'customerCreate', 'creditCardDataSet', 'sessionCreate' und 'transactionPurchase'
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $token (default=null)  Token. Wird bei der IFramelösung erstellt.
		 * @param string $firstname (default=null)  Vorname des Kunden
		 * @param string $surname (default=null)  Nachname des Kunde
		 * @param integer $doublet (default=1)  Dublette erlauben, d.h. eine Kreditkartennummer kann von mehreren Customern verwendet werden. Wenn 0, dann scheitert die Funktion, siehe code 4105
		 * @param string $email (default=null)  E-Mail-Adresse des Kunden, wenn nach den Transaktionen einen E-Mail an der Kunden versand werden soll
		 * @param string $culture (default='de-DE')  Sprache & Land des Kunden | gültige Beispielwerte sind 'de', 'de-DE', 'en-US'
		 * @param string $customerId (default=null)  eigene eindeutige ID des Kunden, wird anderenfalls erzeugt [min./max. Zeichen 10/40, alphanumerisch]
		 * @param map $freeParams (default=null)  Liste mit freien Parametern, die dem Kunden zugeordnet werden
		 * @param string $project das Projektkürzel für den Vorgang
		 * @param string $projectCampaign (default=null)  ein Kampagnenkürzel des Projektbetreibers
		 * @param string $account (default=null)  Account des beteiligten Webmasters sonst eigener - setzt eine Aktivierung der Webmasterfähigkeit des Projekts vorraus - Hinweis: Webmasterfähigkeit steht momentan nicht zur Verfügung
		 * @param string $webmasterCampaign (default=null)  ein Kampagnenkürzel des Webmasters
		 * @param integer $amount (default=null)  abzurechnender Betrag, wird kein Betrag übergeben, wird der Betrag aus der Konfiguration verwendet
		 * @param string $currency (default='EUR')  Währung
		 * @param string $title (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung in Falle einer auftretenden Benachrichtigung wird dieser Wert als Produktidentifizierung mit geschickt, wird kein Wert übergeben, wird Der aus der Konfiguration verwendet
		 * @param string $paytext (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung beim Mailversand, sollten Sie Diesen wünschen
		 * @param string $ip IPv4 oder IPv6 des Benutzers
		 * @param boolean $sendMail (default=true) 
		 * @param map $freeParamsSession (default=null)  Liste mit freien Parametern, die dem Vorgang zugeordnet werden
		 * @param string $sessionId (default=null)  eigene eindeutige ID des Vorgangs, wird anderenfalls erzeugt [max. 40 Zeichen]
		 * @param string $cvc2 (default=null)  CVC2-Code der Kreditkarte. Wenn leer, wird der CVC2 genommen, der bei der Tokenisation abgefragt wurde.
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * @param integer $avsCheck (default=0)  de/aktiviert das AddressVerificationSystem
		 * @param integer $update (default=0)  bei bestehendem Customer: 0=error, 1=update aller Parameter, 2=update nur von gesetzten Parametern
		 * @param integer $recurring (default=0)  Wenn 1 kann der Vorgang als Teil einer Zahlungsfolge markiert werden.
		 * @param integer $start3d (default=0)  Wenn 1 wird der 3dSecure Vorgang gestartet. Entsprechender Response muss beachtet werden.
		 * @param string $returnUrl (default=null)   Url an die nach Abschluß des 3D-Secure Verfahrens weitergeleitet werden soll.
		 * 
		 * @return struct 
		 * @result string $customerId eigene oder erzeugte eindeutige ID des Kunden
		 * @result string $sessionId eigene oder erzeugte eindeutige ID des Vorgangs
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 * @result AVSResult $avsResult (default=null)  wenn ein AVS-Check angefordert wurde dessen Ergebnis
		 * @result string $secureStatus BILLING | START3D | DECLINED | FAILURE
		 * @result string $forwardUrl (default=null)  Wenn gesetzt, muß der Kunde an diese Adresse weitergeleitet werden um daß 3D-Secure Verfahren zu durchlaufen.
		 */
		public function tokenPurchase($accessKey, $testMode=0, $token=null, $firstname=null, $surname=null, $doublet=1, $email=null, $culture='de-DE', $customerId=null, $freeParams=null, $project, $projectCampaign=null, $account=null, $webmasterCampaign=null, $amount=null, $currency='EUR', $title=null, $paytext=null, $ip, $sendMail=true, $freeParamsSession=null, $sessionId=null, $cvc2=null, $fraudDetection=1, $avsCheck=0, $update=0, $recurring=0, $start3d=0, $returnUrl=null);

		/**
		 * sofortige Vorautorisierung
		 *  Kombiniert die Funktionen 'customerCreate', 'creditCardDataSet', 'sessionCreate' und 'transactionAuthorization'
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $token (default=null)  Token. Wird bei der IFramelösung erstellt.
		 * @param string $firstname (default=null)  Vorname des Kunden
		 * @param string $surname (default=null)  Nachname des Kunde
		 * @param integer $doublet (default=1)  Dublette erlauben, d.h. eine Kreditkartennummer kann von mehreren Customern verwendet werden. Wenn 0, dann scheitert die Funktion, siehe code 4105
		 * @param string $email (default=null)  E-Mail-Adresse des Kunden, wenn nach den Transaktionen einen E-Mail an der Kunden versand werden soll
		 * @param string $culture (default='de-DE')  Sprache & Land des Kunden | gültige Beispielwerte sind 'de', 'de-DE', 'en-US'
		 * @param string $customerId (default=null)  eigene eindeutige ID des Kunden, wird anderenfalls erzeugt [min./max. Zeichen 10/40, alphanumerisch]
		 * @param map $freeParams (default=null)  Liste mit freien Parametern, die dem Kunden zugeordnet werden
		 * @param string $project das Projektkürzel für den Vorgang
		 * @param string $projectCampaign (default=null)  ein Kampagnenkürzel des Projektbetreibers
		 * @param string $account (default=null)  Account des beteiligten Webmasters sonst eigener - setzt eine Aktivierung der Webmasterfähigkeit des Projekts vorraus - Hinweis: Webmasterfähigkeit steht momentan nicht zur Verfügung
		 * @param string $webmasterCampaign (default=null)  ein Kampagnenkürzel des Webmasters
		 * @param integer $amount (default=null)  abzurechnender Betrag, wird kein Betrag übergeben, wird der Betrag aus der Konfiguration verwendet
		 * @param string $currency (default='EUR')  Währung
		 * @param string $title (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung in Falle einer auftretenden Benachrichtigung wird dieser Wert als Produktidentifizierung mit geschickt, wird kein Wert übergeben, wird Der aus der Konfiguration verwendet
		 * @param string $paytext (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung beim Mailversand, sollten Sie Diesen wünschen
		 * @param string $ip IPv4 oder IPv6 des Benutzers
		 * @param boolean $sendMail (default=true) 
		 * @param map $freeParamsSession (default=null)  Liste mit freien Parametern, die dem Vorgang zugeordnet werden
		 * @param string $sessionId (default=null)  eigene eindeutige ID des Vorgangs, wird anderenfalls erzeugt [max. 40 Zeichen]
		 * @param string $cvc2 (default=null)  CVC2-Code der Kreditkarte, muß min einmal pro Kreditkarte/Verfallszeit angegeben worden sein
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * @param integer $avsCheck (default=0)  de/aktiviert das AddressVerificationSystem
		 * @param integer $update (default=0)  bei bestehendem Customer: 0=error, 1=update aller Parameter, 2=update nur von gesetzten Parametern
		 * @param integer $start3d (default=0)  Wenn 1 wird der 3dSecure Vorgang gestartet. Entsprechender Response muss beachtet werden.
		 * @param string $returnUrl (default=null)   Url an die nach Abschluß des 3D-Secure Verfahrens weitergeleitet werden soll.
		 * 
		 * @return struct 
		 * @result string $customerId eigene oder erzeugte eindeutige ID des Kunden
		 * @result string $sessionId eigene oder erzeugte eindeutige ID des Vorgangs
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 * @result AVSResult $avsResult (default=null)  wenn ein AVS-Check angefordert wurde dessen Ergebnis
		 * @result string $secureStatus BILLING | START3D | DECLINED | FAILURE
		 * @result string $forwardUrl (default=null)  Wenn gesetzt, muß der Kunde an diese Adresse weitergeleitet werden um daß 3D-Secure Verfahren zu durchlaufen.
		 */
		public function tokenAuthorization($accessKey, $testMode=0, $token=null, $firstname=null, $surname=null, $doublet=1, $email=null, $culture='de-DE', $customerId=null, $freeParams=null, $project, $projectCampaign=null, $account=null, $webmasterCampaign=null, $amount=null, $currency='EUR', $title=null, $paytext=null, $ip, $sendMail=true, $freeParamsSession=null, $sessionId=null, $cvc2=null, $fraudDetection=1, $avsCheck=0, $update=0, $start3d=0, $returnUrl=null);

		/**
		 * Gibt Informationen über einen Token zurück. Dieser ensteht bei der Tokenisation mit der JsBridge.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $token Token. Wird bei der IFramelösung erstellt.
		 * 
		 * @return struct 
		 * @result string $holder Kreditkartenholder
		 * @result string $type Kartentyp
		 * @result string $number maskierte Kreditkartennummer
		 * @result integer $expiryYear Gültigkeits Jahr
		 * @result integer $expiryMonth Gültigkeits Monat
		 * @result string $cvc maskierter CVC-Code
		 * @result string $ip IP des Tokenerstellers
		 * @result datetime $expires maximale Gültigkeit des Tokens
		 */
		public function tokenGet($accessKey, $testMode=0, $token);

		/**
		 * sofortige Buchung. Benötigt einen bestehenden Customer, bevorzugt für Folgebuchungen geeignet.
		 *  Kombiniert die Funktionen 'sessionCreate' und 'transactionPurchase'
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * @param string $sessionId (default=null)  eigene eindeutige ID des Vorgangs, wird anderenfalls erzeugt [max. 40 Zeichen]
		 * @param string $project das Projektkürzel für den Vorgang
		 * @param string $cvc2 (default=null)  CVC2-Code der Kreditkarte, muß min einmal pro Kreditkarte/Verfallszeit angegeben worden sein
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * @param integer $avsCheck (default=0)  de/aktiviert das AddressVerificationSystem
		 * @param string $projectCampaign (default=null)  ein Kampagnenkürzel des Projektbetreibers
		 * @param string $account (default=null)  Account des beteiligten Webmasters sonst eigener - setzt eine Aktivierung der Webmasterfähigkeit des Projekts vorraus - Hinweis: Webmasterfähigkeit steht momentan nicht zur Verfügung
		 * @param string $webmasterCampaign (default=null)  ein Kampagnenkürzel des Webmasters
		 * @param integer $amount (default=null)  abzurechnender Betrag, wird kein Betrag übergeben, wird der Betrag aus der Konfiguration verwendet
		 * @param string $currency (default='EUR')  Währung
		 * @param string $title (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung in Falle einer auftretenden Benachrichtigung wird dieser Wert als Produktidentifizierung mit geschickt, wird kein Wert übergeben, wird Der aus der Konfiguration verwendet
		 * @param string $paytext (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung beim Mailversand, sollten Sie Diesen wünschen
		 * @param string $ip IPv4 oder IPv6 des Benutzers
		 * @param map $freeParamsSession (default=null)  Liste mit freien Parametern, die dem Vorgang zugeordnet werden
		 * @param boolean $sendMail (default=true) 
		 * @param integer $recurring (default=0)  Wenn 1 kann der Vorgang als Teil einer Zahlungsfolge markiert werden.
		 * 
		 * @return struct 
		 * @result string $sessionId eigene oder erzeugte eindeutige ID des Vorgangs
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 * @result AVSResult $avsResult (default=null)  wenn ein AVS-Check angefordert wurde dessen Ergebnis
		 */
		public function shortCustomerPurchase($accessKey, $testMode=0, $customerId, $sessionId=null, $project, $cvc2=null, $fraudDetection=1, $avsCheck=0, $projectCampaign=null, $account=null, $webmasterCampaign=null, $amount=null, $currency='EUR', $title=null, $paytext=null, $ip, $freeParamsSession=null, $sendMail=true, $recurring=0);

		/**
		 * sofortige Buchung
		 *  Kombiniert die Funktionen 'customerCreate', 'creditCardDataSet', 'sessionCreate' und 'transactionPurchase'
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $firstname (default=null)  Vorname des Kunden
		 * @param string $surname (default=null)  Nachname des Kunde
		 * @param string $number Kreditkartennummer
		 * @param integer $expiryYear Gültigkeits Jahr
		 * @param integer $expiryMonth Gültigkeits Monat
		 * @param integer $doublet (default=1)  Dublette erlauben, d.h. eine Kreditkartennummer kann von mehreren Customern verwendet werden. Wenn 0, dann scheitert die Funktion, siehe code 4105
		 * @param string $email (default=null)  E-Mail-Adresse des Kunden, wenn nach den Transaktionen einen E-Mail an der Kunden versand werden soll
		 * @param string $culture (default='de-DE')  Sprache & Land des Kunden | gültige Beispielwerte sind 'de', 'de-DE', 'en-US'
		 * @param string $customerId (default=null)  eigene eindeutige ID des Kunden, wird anderenfalls erzeugt [min./max. Zeichen 10/40, alphanumerisch]
		 * @param map $freeParams (default=null)  Liste mit freien Parametern, die dem Kunden zugeordnet werden
		 * @param string $project das Projektkürzel für den Vorgang
		 * @param string $projectCampaign (default=null)  ein Kampagnenkürzel des Projektbetreibers
		 * @param string $account (default=null)  Account des beteiligten Webmasters sonst eigener - setzt eine Aktivierung der Webmasterfähigkeit des Projekts vorraus - Hinweis: Webmasterfähigkeit steht momentan nicht zur Verfügung
		 * @param string $webmasterCampaign (default=null)  ein Kampagnenkürzel des Webmasters
		 * @param integer $amount (default=null)  abzurechnender Betrag, wird kein Betrag übergeben, wird der Betrag aus der Konfiguration verwendet
		 * @param string $currency (default='EUR')  Währung
		 * @param string $title (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung in Falle einer auftretenden Benachrichtigung wird dieser Wert als Produktidentifizierung mit geschickt, wird kein Wert übergeben, wird Der aus der Konfiguration verwendet
		 * @param string $paytext (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung beim Mailversand, sollten Sie Diesen wünschen
		 * @param string $ip IPv4 oder IPv6 des Benutzers
		 * @param boolean $sendMail (default=true) 
		 * @param map $freeParamsSession (default=null)  Liste mit freien Parametern, die dem Vorgang zugeordnet werden
		 * @param string $sessionId (default=null)  eigene eindeutige ID des Vorgangs, wird anderenfalls erzeugt [max. 40 Zeichen]
		 * @param string $cvc2 (default=null)  CVC2-Code der Kreditkarte, muß min einmal pro Kreditkarte/Verfallszeit angegeben worden sein
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * @param integer $avsCheck (default=0)  de/aktiviert das AddressVerificationSystem
		 * @param integer $update (default=0)  bei bestehendem Customer: 0=error, 1=update aller Parameter, 2=update nur von gesetzten Parametern
		 * @param integer $recurring (default=0)  Wenn 1 kann der Vorgang als Teil einer Zahlungsfolge markiert werden.
		 * 
		 * @return struct 
		 * @result string $customerId eigene oder erzeugte eindeutige ID des Kunden
		 * @result string $sessionId eigene oder erzeugte eindeutige ID des Vorgangs
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 * @result AVSResult $avsResult (default=null)  wenn ein AVS-Check angefordert wurde dessen Ergebnis
		 */
		public function shortTransactionPurchase($accessKey, $testMode=0, $firstname=null, $surname=null, $number, $expiryYear, $expiryMonth, $doublet=1, $email=null, $culture='de-DE', $customerId=null, $freeParams=null, $project, $projectCampaign=null, $account=null, $webmasterCampaign=null, $amount=null, $currency='EUR', $title=null, $paytext=null, $ip, $sendMail=true, $freeParamsSession=null, $sessionId=null, $cvc2=null, $fraudDetection=1, $avsCheck=0, $update=0, $recurring=0);

		/**
		 * Führt eine Transaktion zur Buchung (Gesamt- oder Teilbetrag) einer Vorautorisierung durch.
		 * 
		 *  Sie buchen den reservierten Betrag:
		 *  - In voller Höhe (Gesamtbetrag) - der Parameter `amount` entfällt ([NULL])
		 *  - Oder nur einen Teilbetrag - der Parameter `amount` enthält den gewünschten Teilbetrag
		 * 
		 *  Zur Durchführung der Transaktion ist es notwendig das Sie mittels `sessionCreate` einen Bezahlvorgang,
		 *  sowie unter Verwendung von `transactionAuthorization` eine Vorautorisierung, erstellt haben.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * @param string $transactionId (default=null)  Transaktionsnummer von "transactionAuthorization"
		 * @param integer $amount (default=null)  [NULL] - entspricht Betrag aus Vorautorisierung | wenn abweichend, der zu buchende Betrag <= Betrag aus Vorautorisierung
		 * @param string $currency (default=null)  Währung, falls abweichend von Originaltransaktion [nur relevant wenn `amount` abweichend]
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * @param boolean $sendMail (default=null)  wenn gesetzt, überschreibt er den Wert der bei sessionCreate gesetzt wurde
		 * 
		 * @return struct 
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 */
		public function transactionCapture($accessKey, $testMode=0, $sessionId, $transactionId=null, $amount=null, $currency=null, $fraudDetection=1, $sendMail=null);

		/**
		 * Gibt Informationen über eine Transaktion anhand einer verschlüsselten Transaktionsnummer zurück. Diese ensteht bei der Tokenisation mit der JsBridge.
		 * 
		 *  Wenn sich `currencyInit` und `currencyBooked` unterscheiden, dann fehlt die native Unterstützung für die gewünschte Währung.
		 *  In diesem Fall erfolgte eine Umrechnung in eine nativ unterstützte Währung.
		 *  Die verschlüsselte Transaktionsnummer, $cryptId, wird an die Rücksprungadresse nach dem Zahlvorgang angehängt, siehe IFrameScript Billingfunktionen.
		 *  Für die weiteren Abfragen etc. nutzten Sie bitte die Transaktionsnummer, $transactionId, aus dem Response, da die $cryptId ca. 2 Stunden nach
		 *  der Erstellung ungültig wird!
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $cryptId verschlüsselte Transaktionsnummer
		 * 
		 * @return struct 
		 * @result string $transactionId Transaktionsnummer
		 * @result string $sessionId eindeutige ID des Vorgangs
		 * @result string $customerId ID des Kunden
		 * @result string $auth AuthCode
		 * @result integer $amountInit Betrag der abgerechnet werden sollte in currencyInit
		 * @result string $currencyInit Währungseinheit mit der abgerechnet werden sollte
		 * @result integer $amountBooked gebuchter Betrag in currencyBooked
		 * @result string $currencyBooked Währungseinheit mit der gebucht wurde
		 * @result integer $amountInternal abgerechneter Betrag in currencyInternal
		 * @result string $currencyInternal (default='EUR')  Währungseinheit für interne Verbuchung
		 * @result TransactionType $type Art der Transaktion
		 * @result TransactionStatus $status Status der Transaktion
		 * @result datetime $created Zeitpunkt der Transaktion
		 * @result string $ip IPv4 oder IPv6 des Benutzers
		 * @result string $cardType (default=null)  Kartentyp, kann bei aktiviertem DataStorage-Feature null sein
		 * @result string $cardNumber (default=null)  partielle Kreditkartennummer [letzten 4 Stellen], kann bei aktiviertem DataStorage-Feature null sein
		 * @result integer $cardExpiryYear (default=null)  Gültigkeits Jahr, kann bei aktiviertem DataStorage-Feature null sein
		 * @result integer $cardExpiryMonth (default=null) 	Gültigkeits Monat, kann bei aktiviertem DataStorage-Feature null sein
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 * @result AVSResult $avsResult (default=null)  Wenn ein AVS-Check durchgeführt wurde dessen Ergebnis
		 */
		public function transactionGetCrypt($accessKey, $testMode=0, $cryptId);

		/**
		 * Erstellt einen neuen Kunden und verknüpft Kreditkartendaten.
		 *  Entspricht der Kombination aus 'customerCreate' und 'creditCardDataSet', wobei die benötigten Daten aus dem token generiert werden.
		 *  Der token wird bei der Tokenisation über das IFrameScript erzeugt.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $token (default=null)  Token. Wird bei der IFramelösung erstellt.
		 * @param string $firstname (default=null)  Vorname des Kunden
		 * @param string $surname (default=null)  Nachname des Kunde
		 * @param integer $doublet (default=1)  Dublette erlauben, d.h. eine Kreditkartennummer kann von mehreren Customern verwendet werden. Wenn 0, dann scheitert die Funktion, siehe code 4105
		 * @param string $email (default=null)  E-Mail-Adresse des Kunden, wenn nach den Transaktionen einen E-Mail an der Kunden versand werden soll
		 * @param string $culture (default='de-DE')  Sprache & Land des Kunden | gültige Beispielwerte sind 'de', 'de-DE', 'en-US'
		 * @param string $customerId (default=null)  eigene eindeutige ID des Kunden, wird anderenfalls erzeugt [min./max. Zeichen 10/40, alphanumerisch]
		 * @param map $freeParams (default=null)  Liste mit freien Parametern, die dem Kunden zugeordnet werden
		 * @param integer $update (default=0)  bei bestehendem Customer: 0=error, 1=update aller Parameter, 2=update nur von gesetzten Parametern
		 * 
		 * @return struct 
		 * @result string $customerId eigene oder erzeugte eindeutige ID des Kunden
		 * @result boolean $cvc2Required gibt Auskunft darüber, ob bei der nächsten Buchung der CVC2-Code erforderlich ist
		 */
		public function tokenCustomerCreate($accessKey, $testMode=0, $token=null, $firstname=null, $surname=null, $doublet=1, $email=null, $culture='de-DE', $customerId=null, $freeParams=null, $update=0);

		/**
		 * Erzeugt einen neuen Bezahlvorgang für einen Kunden. Der Kunde und die Session werden angelegt.
		 *  Kombiniert die Funktionen 'tokenCustomerCreate', und 'sessionCreate'
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $token (default=null)  Token. Wird bei der IFramelösung erstellt.
		 * @param string $firstname (default=null)  Vorname des Kunden
		 * @param string $surname (default=null)  Nachname des Kunde
		 * @param integer $doublet (default=1)  Dublette erlauben, d.h. eine Kreditkartennummer kann von mehreren Customern verwendet werden. Wenn 0, dann scheitert die Funktion, siehe code 4105
		 * @param string $email (default=null)  E-Mail-Adresse des Kunden, wenn nach den Transaktionen einen E-Mail an der Kunden versand werden soll
		 * @param string $culture (default='de-DE')  Sprache & Land des Kunden | gültige Beispielwerte sind 'de', 'de-DE', 'en-US'
		 * @param string $customerId (default=null)  eigene eindeutige ID des Kunden, wird anderenfalls erzeugt [min./max. Zeichen 10/40, alphanumerisch]
		 * @param map $freeParams (default=null)  Liste mit freien Parametern, die dem Kunden zugeordnet werden
		 * @param string $project das Projektkürzel für den Vorgang
		 * @param string $projectCampaign (default=null)  ein Kampagnenkürzel des Projektbetreibers
		 * @param string $account (default=null)  Account des beteiligten Webmasters sonst eigener - setzt eine Aktivierung der Webmasterfähigkeit des Projekts vorraus - Hinweis: Webmasterfähigkeit steht momentan nicht zur Verfügung
		 * @param string $webmasterCampaign (default=null)  ein Kampagnenkürzel des Webmasters
		 * @param integer $amount (default=null)  abzurechnender Betrag, wird kein Betrag übergeben, wird der Betrag aus der Konfiguration verwendet
		 * @param string $currency (default='EUR')  Währung
		 * @param string $title (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung in Falle einer auftretenden Benachrichtigung wird dieser Wert als Produktidentifizierung mit geschickt, wird kein Wert übergeben, wird Der aus der Konfiguration verwendet
		 * @param string $paytext (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung beim Mailversand, sollten Sie Diesen wünschen
		 * @param string $ip IPv4 oder IPv6 des Benutzers
		 * @param boolean $sendMail (default=true) 
		 * @param map $freeParamsSession (default=null)  Liste mit freien Parametern, die dem Vorgang zugeordnet werden
		 * @param string $sessionId (default=null)  eigene eindeutige ID des Vorgangs, wird anderenfalls erzeugt [max. 40 Zeichen]
		 * @param integer $update (default=0)  bei bestehendem Customer: 0=error, 1=update aller Parameter, 2=update nur von gesetzten Parametern
		 * 
		 * @return struct 
		 * @result string $customerId eigene oder erzeugte eindeutige ID des Kunden
		 * @result boolean $cvc2Required gibt Auskunft darüber, ob bei der nächsten Buchung der CVC2-Code erforderlich ist
		 * @result string $sessionId eigene oder erzeugte eindeutige ID des Vorgangs
		 * @result SessionStatus $status Vorgangsstatus "INIT"
		 * @result datetime $expire Ablaufzeit des Vorgangs
		 */
		public function tokenSessionCreate($accessKey, $testMode=0, $token=null, $firstname=null, $surname=null, $doublet=1, $email=null, $culture='de-DE', $customerId=null, $freeParams=null, $project, $projectCampaign=null, $account=null, $webmasterCampaign=null, $amount=null, $currency='EUR', $title=null, $paytext=null, $ip, $sendMail=true, $freeParamsSession=null, $sessionId=null, $update=0);

		/**
		 * Erstellt einen neuen Kunden und verknüpft Kreditkartendaten.
		 *  Kombiniert die Funktionen 'customerCreate' und 'creditCardDataSet'
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $firstname (default=null)  Vorname des Kunden
		 * @param string $surname (default=null)  Nachname des Kunde
		 * @param string $number Kreditkartennummer
		 * @param integer $expiryYear Gültigkeits Jahr
		 * @param integer $expiryMonth Gültigkeits Monat
		 * @param integer $doublet (default=1)  Dublette erlauben, d.h. eine Kreditkartennummer kann von mehreren Customern verwendet werden. Wenn 0, dann scheitert die Funktion, siehe code 4105
		 * @param string $email (default=null)  E-Mail-Adresse des Kunden, wenn nach den Transaktionen einen E-Mail an der Kunden versand werden soll
		 * @param string $culture (default='de-DE')  Sprache & Land des Kunden | gültige Beispielwerte sind 'de', 'de-DE', 'en-US'
		 * @param string $customerId (default=null)  eigene eindeutige ID des Kunden, wird anderenfalls erzeugt [min./max. Zeichen 10/40, alphanumerisch]
		 * @param map $freeParams (default=null)  Liste mit freien Parametern, die dem Kunden zugeordnet werden
		 * @param integer $update (default=0)  bei bestehendem Customer: 0=error, 1=update aller Parameter, 2=update nur von gesetzten Parametern
		 * 
		 * @return struct 
		 * @result string $customerId eigene oder erzeugte eindeutige ID des Kunden
		 * @result boolean $cvc2Required gibt Auskunft darüber, ob bei der nächsten Buchung der CVC2-Code erforderlich ist
		 */
		public function shortCustomerCreate($accessKey, $testMode=0, $firstname=null, $surname=null, $number, $expiryYear, $expiryMonth, $doublet=1, $email=null, $culture='de-DE', $customerId=null, $freeParams=null, $update=0);

		/**
		 * Erzeugt einen neuen Bezahlvorgang für einen Kunden. Der Kunde und die Session werden angelegt.
		 *  Kombiniert die Funktionen 'customerCreate', 'creditCardDataSet' und 'sessionCreate'
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $firstname (default=null)  Vorname des Kunden
		 * @param string $surname (default=null)  Nachname des Kunde
		 * @param string $number Kreditkartennummer
		 * @param integer $expiryYear Gültigkeits Jahr
		 * @param integer $expiryMonth Gültigkeits Monat
		 * @param integer $doublet (default=1)  Dublette erlauben, d.h. eine Kreditkartennummer kann von mehreren Customern verwendet werden. Wenn 0, dann scheitert die Funktion, siehe code 4105
		 * @param string $email (default=null)  E-Mail-Adresse des Kunden, wenn nach den Transaktionen einen E-Mail an der Kunden versand werden soll
		 * @param string $culture (default='de-DE')  Sprache & Land des Kunden | gültige Beispielwerte sind 'de', 'de-DE', 'en-US'
		 * @param string $customerId (default=null)  eigene eindeutige ID des Kunden, wird anderenfalls erzeugt [min./max. Zeichen 10/40, alphanumerisch]
		 * @param map $freeParams (default=null)  Liste mit freien Parametern, die dem Kunden zugeordnet werden
		 * @param string $project das Projektkürzel für den Vorgang
		 * @param string $projectCampaign (default=null)  ein Kampagnenkürzel des Projektbetreibers
		 * @param string $account (default=null)  Account des beteiligten Webmasters sonst eigener - setzt eine Aktivierung der Webmasterfähigkeit des Projekts vorraus - Hinweis: Webmasterfähigkeit steht momentan nicht zur Verfügung
		 * @param string $webmasterCampaign (default=null)  ein Kampagnenkürzel des Webmasters
		 * @param integer $amount (default=null)  abzurechnender Betrag, wird kein Betrag übergeben, wird der Betrag aus der Konfiguration verwendet
		 * @param string $currency (default='EUR')  Währung
		 * @param string $title (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung in Falle einer auftretenden Benachrichtigung wird dieser Wert als Produktidentifizierung mit geschickt, wird kein Wert übergeben, wird Der aus der Konfiguration verwendet
		 * @param string $paytext (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung beim Mailversand, sollten Sie Diesen wünschen
		 * @param string $ip IPv4 oder IPv6 des Benutzers
		 * @param boolean $sendMail (default=true) 
		 * @param map $freeParamsSession (default=null)  Liste mit freien Parametern, die dem Vorgang zugeordnet werden
		 * @param string $sessionId (default=null)  eigene eindeutige ID des Vorgangs, wird anderenfalls erzeugt [max. 40 Zeichen]
		 * @param integer $update (default=0)  bei bestehendem Customer: 0=error, 1=update aller Parameter, 2=update nur von gesetzten Parametern
		 * 
		 * @return struct 
		 * @result string $customerId eigene oder erzeugte eindeutige ID des Kunden
		 * @result boolean $cvc2Required gibt Auskunft darüber, ob bei der nächsten Buchung der CVC2-Code erforderlich ist
		 * @result string $sessionId eigene oder erzeugte eindeutige ID des Vorgangs
		 * @result SessionStatus $status Vorgangsstatus "INIT"
		 * @result datetime $expire Ablaufzeit des Vorgangs
		 */
		public function shortSessionCreate($accessKey, $testMode=0, $firstname=null, $surname=null, $number, $expiryYear, $expiryMonth, $doublet=1, $email=null, $culture='de-DE', $customerId=null, $freeParams=null, $project, $projectCampaign=null, $account=null, $webmasterCampaign=null, $amount=null, $currency='EUR', $title=null, $paytext=null, $ip, $sendMail=true, $freeParamsSession=null, $sessionId=null, $update=0);

		/**
		 * sofortige Vorautorisierung
		 *  Kombiniert die Funktionen 'customerCreate', 'creditCardDataSet', 'sessionCreate' und 'transactionAuthorization'
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $firstname (default=null)  Vorname des Kunden
		 * @param string $surname (default=null)  Nachname des Kunde
		 * @param string $number Kreditkartennummer
		 * @param integer $expiryYear Gültigkeits Jahr
		 * @param integer $expiryMonth Gültigkeits Monat
		 * @param integer $doublet (default=1)  Dublette erlauben, d.h. eine Kreditkartennummer kann von mehreren Customern verwendet werden. Wenn 0, dann scheitert die Funktion, siehe code 4105
		 * @param string $email (default=null)  E-Mail-Adresse des Kunden, wenn nach den Transaktionen einen E-Mail an der Kunden versand werden soll
		 * @param string $culture (default='de-DE')  Sprache & Land des Kunden | gültige Beispielwerte sind 'de', 'de-DE', 'en-US'
		 * @param string $customerId (default=null)  eigene eindeutige ID des Kunden, wird anderenfalls erzeugt [min./max. Zeichen 10/40, alphanumerisch]
		 * @param map $freeParams (default=null)  Liste mit freien Parametern, die dem Kunden zugeordnet werden
		 * @param string $project das Projektkürzel für den Vorgang
		 * @param string $projectCampaign (default=null)  ein Kampagnenkürzel des Projektbetreibers
		 * @param string $account (default=null)  Account des beteiligten Webmasters sonst eigener - setzt eine Aktivierung der Webmasterfähigkeit des Projekts vorraus - Hinweis: Webmasterfähigkeit steht momentan nicht zur Verfügung
		 * @param string $webmasterCampaign (default=null)  ein Kampagnenkürzel des Webmasters
		 * @param integer $amount (default=null)  abzurechnender Betrag, wird kein Betrag übergeben, wird der Betrag aus der Konfiguration verwendet
		 * @param string $currency (default='EUR')  Währung
		 * @param string $title (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung in Falle einer auftretenden Benachrichtigung wird dieser Wert als Produktidentifizierung mit geschickt, wird kein Wert übergeben, wird Der aus der Konfiguration verwendet
		 * @param string $paytext (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung beim Mailversand, sollten Sie Diesen wünschen
		 * @param string $ip IPv4 oder IPv6 des Benutzers
		 * @param boolean $sendMail (default=true) 
		 * @param map $freeParamsSession (default=null)  Liste mit freien Parametern, die dem Vorgang zugeordnet werden
		 * @param string $sessionId (default=null)  eigene eindeutige ID des Vorgangs, wird anderenfalls erzeugt [max. 40 Zeichen]
		 * @param string $cvc2 (default=null)  CVC2-Code der Kreditkarte, muß min einmal pro Kreditkarte/Verfallszeit angegeben worden sein
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * @param integer $avsCheck (default=0)  de/aktiviert das AddressVerificationSystem
		 * @param integer $update (default=0)  bei bestehendem Customer: 0=error, 1=update aller Parameter, 2=update nur von gesetzten Parametern
		 * 
		 * @return struct 
		 * @result string $customerId eigene oder erzeugte eindeutige ID des Kunden
		 * @result string $sessionId eigene oder erzeugte eindeutige ID des Vorgangs
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 * @result AVSResult $avsResult (default=null)  wenn ein AVS-Check angefordert wurde dessen Ergebnis
		 */
		public function shortTransactionAuthorization($accessKey, $testMode=0, $firstname=null, $surname=null, $number, $expiryYear, $expiryMonth, $doublet=1, $email=null, $culture='de-DE', $customerId=null, $freeParams=null, $project, $projectCampaign=null, $account=null, $webmasterCampaign=null, $amount=null, $currency='EUR', $title=null, $paytext=null, $ip, $sendMail=true, $freeParamsSession=null, $sessionId=null, $cvc2=null, $fraudDetection=1, $avsCheck=0, $update=0);

		/**
		 * sofortige Vorautorisierung. Benötigt einen bestehenden Customer, bevorzugt für Folgebuchungen geeignet.
		 *  Kombiniert die Funktionen 'sessionCreate' und 'transactionAuthorization'
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * @param string $sessionId (default=null)  eigene eindeutige ID des Vorgangs, wird anderenfalls erzeugt [max. 40 Zeichen]
		 * @param string $project das Projektkürzel für den Vorgang
		 * @param string $cvc2 (default=null)  CVC2-Code der Kreditkarte, muß min einmal pro Kreditkarte/Verfallszeit angegeben worden sein
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * @param integer $avsCheck (default=0)  de/aktiviert das AddressVerificationSystem
		 * @param string $projectCampaign (default=null)  ein Kampagnenkürzel des Projektbetreibers
		 * @param string $account (default=null)  Account des beteiligten Webmasters sonst eigener - setzt eine Aktivierung der Webmasterfähigkeit des Projekts vorraus - Hinweis: Webmasterfähigkeit steht momentan nicht zur Verfügung
		 * @param string $webmasterCampaign (default=null)  ein Kampagnenkürzel des Webmasters
		 * @param integer $amount (default=null)  abzurechnender Betrag, wird kein Betrag übergeben, wird der Betrag aus der Konfiguration verwendet
		 * @param string $currency (default='EUR')  Währung
		 * @param string $title (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung in Falle einer auftretenden Benachrichtigung wird dieser Wert als Produktidentifizierung mit geschickt, wird kein Wert übergeben, wird Der aus der Konfiguration verwendet
		 * @param string $paytext (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung beim Mailversand, sollten Sie Diesen wünschen
		 * @param string $ip IPv4 oder IPv6 des Benutzers
		 * @param map $freeParamsSession (default=null)  Liste mit freien Parametern, die dem Vorgang zugeordnet werden
		 * @param boolean $sendMail (default=true) 
		 * 
		 * @return struct 
		 * @result string $sessionId eigene oder erzeugte eindeutige ID des Vorgangs
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 * @result AVSResult $avsResult (default=null)  wenn ein AVS-Check angefordert wurde dessen Ergebnis
		 */
		public function shortCustomerAuthorization($accessKey, $testMode=0, $customerId, $sessionId=null, $project, $cvc2=null, $fraudDetection=1, $avsCheck=0, $projectCampaign=null, $account=null, $webmasterCampaign=null, $amount=null, $currency='EUR', $title=null, $paytext=null, $ip, $freeParamsSession=null, $sendMail=true);

		/**
		 * Erstellt einen neuen Kunden, überschreibt ihn wenn gewünscht.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId (default=null)  eigene eindeutige ID des Kunden, wird anderenfalls erzeugt [min./max. Zeichen 10/40, alphanumerisch]
		 * @param map $freeParams (default=null)  Liste mit freien Parametern, die dem Kunden zugeordnet werden
		 * @param string $firstname (default=null)  Vorname des Kunden
		 * @param string $surname (default=null)  Nachname des Kunde
		 * @param string $email (default=null)  E-Mail-Adresse des Kunden, wenn nach den Transaktionen einen E-Mail an der Kunden versand werden soll
		 * @param string $culture (default='de-DE')  Sprache & Land des Kunden | gültige Beispielwerte sind 'de', 'de-DE', 'en-US'
		 * @param integer $update (default=0)  bei bestehendem Customer: 0=error, 1=update aller Parameter, 2=update nur von gesetzten Parametern
		 * 
		 * @return string eigene oder erzeugte eindeutige ID des Kunden
		 */
		public function customerCreate($accessKey, $testMode=0, $customerId=null, $freeParams=null, $firstname=null, $surname=null, $email=null, $culture='de-DE', $update=0);

		/**
		 * Verknüpft Kreditkartendaten mit einem bestehenden Kunden
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * @param string $number Kreditkartennummer
		 * @param integer $expiryYear Gültigkeits Jahr
		 * @param integer $expiryMonth Gültigkeits Monat
		 * @param integer $doublet (default=1)  Dublette erlauben, d.h. eine Kreditkartennummer kann von mehreren Customern verwendet werden. Wenn 0, dann scheitert die Funktion, siehe code 4105
		 * 
		 * @return boolean Rückgabewert gibt Auskunft darüber, ob bei der nächsten Buchung der CVC2-Code erforderlich ist - cvc2Required
		 */
		public function creditcardDataSet($accessKey, $testMode=0, $customerId, $number, $expiryYear, $expiryMonth, $doublet=1);

		/**
		 * Liefert Liste mit bestehenden Kunden.
		 *  Sortiert ist nach Erstellungszeitpunkt, ältester Wert zuerst. Erstellungszeitpunkt ist eindeutig.
		 *  Ein paging kann realisiert werden, indem der letzte Ergebnissatz als customerId Parameter beim Folgeaufruf verwendet wird.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0) 	aktiviert Testumgebung
		 * @param string $customerId (default=null) 	wenn angegeben werden nur Kunden gelistet, deren Erstellungszeitpunkt jünger oder identisch mit dem des angegebenen Kunden ist
		 * @param integer $onlyNewer (default=1) 	wenn 1 dann nur Kunden listen, deren Erstellungszeitpunkt jünger als bei dem angegebenen Kunden ist
		 * @param integer $onlyDoublet (default=0) 	wenn 1 dann nur Kunden listen, die sich mit anderen Kunden eine Kreditkartennummer teilen
		 * @param datetime $from (default=null) 	wenn angegeben nur Kunden listen deren Erstellungszeitpunkt >= dem angegeben Wert ist [YYYY-mm-dd HH:ii:ss]
		 * @param datetime $to (default=null) 	wenn angegeben nur Kunden listen deren Erstellungszeitpunkt < dem angegeben Wert ist [YYYY-mm-dd HH:ii:ss]
		 * @param integer $limit (default=20) 	maximale Anzahl der Liste, maximaler Wert ist 500
		 * 
		 * @return array 
		 * @result integer $count Anzahl Ergebnissätze in customer
		 * @result integer $more wenn 1 dann hat limit gegriffen, d.h. es gibt mehr Ergebnissätze für das verwendete Filter
		 * @result string[] $customer (default=null)  Liste von customerId`s
		 */
		public function customerList($accessKey, $testMode=0, $customerId=null, $onlyNewer=1, $onlyDoublet=0, $from=null, $to=null, $limit=20);

		/**
		 * Liefert Liste mit Kunden die die gleiche Kreditkarte nutzen wie der zu prüfende Kunde.
		 *  Sortiert ist nach Erstellungszeitpunkt, ältester Wert zuerst. Erstellungszeitpunkt ist eindeutig.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0) 	aktiviert Testumgebung
		 * @param string $customerId zu prüfender Kunde
		 * @param datetime $from (default=null) 	wenn angegeben nur Kunden listen deren Erstellungszeitpunkt >= dem angegeben Wert ist [YYYY-mm-dd HH:ii:ss]
		 * @param datetime $to (default=null) 	wenn angegeben nur Kunden listen deren Erstellungszeitpunkt < dem angegeben Wert ist [YYYY-mm-dd HH:ii:ss]
		 * @param integer $limit (default=20) 	maximale Anzahl der Liste, maximaler Wert ist 500
		 * 
		 * @return array 
		 * @result integer $count Anzahl Ergebnissätze in customer
		 * @result integer $more wenn 1 dann hat limit gegriffen, d.h. es gibt mehr Ergebnissätze für das verwendete Filter
		 * @result string[] $customer (default=null)   Liste von customerId`s
		 */
		public function customerDoubletList($accessKey, $testMode=0, $customerId, $from=null, $to=null, $limit=20);

		/**
		 * Gibt Informationen über eine Transaktion anhand einer Transaktionsnummer zurück.
		 * 
		 *  Wenn sich `currencyInit` und `currencyBooked` unterscheiden, dann fehlt die native Unterstützung für die gewünschte Währung.
		 *  In diesem Fall erfolgte eine Umrechnung in eine nativ unterstützte Währung.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $transactionId Transaktionsnummer
		 * 
		 * @return struct 
		 * @result string $transactionId Transaktionsnummer
		 * @result string $sessionId eindeutige ID des Vorgangs
		 * @result string $customerId ID des Kunden
		 * @result string $auth AuthCode
		 * @result integer $amountInit Betrag der abgerechnet werden sollte in currencyInit
		 * @result string $currencyInit Währungseinheit mit der abgerechnet werden sollte
		 * @result integer $amountBooked gebuchter Betrag in currencyBooked
		 * @result string $currencyBooked Währungseinheit mit der gebucht wurde
		 * @result integer $amountInternal abgerechneter Betrag in currencyInternal
		 * @result string $currencyInternal (default='EUR')  Währungseinheit für interne Verbuchung
		 * @result TransactionType $type Art der Transaktion
		 * @result TransactionStatus $status Status der Transaktion
		 * @result datetime $created Zeitpunkt der Transaktion
		 * @result string $ip IPv4 oder IPv6 des Benutzers
		 * @result string $cardType (default=null)  Kartentyp, kann bei aktiviertem DataStorage-Feature null sein
		 * @result string $cardNumber (default=null)  partielle Kreditkartennummer [letzten 4 Stellen], kann bei aktiviertem DataStorage-Feature null sein
		 * @result integer $cardExpiryYear (default=null)  Gültigkeits Jahr, kann bei aktiviertem DataStorage-Feature null sein
		 * @result integer $cardExpiryMonth (default=null) 	Gültigkeits Monat, kann bei aktiviertem DataStorage-Feature null sein
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 * @result AVSResult $avsResult (default=null)  Wenn ein AVS-Check durchgeführt wurde dessen Ergebnis
		 */
		public function transactionGet($accessKey, $testMode=0, $transactionId);

		/**
		 * Versendet eine Benachrichtigung über eine transaktionsbezogene Gutschrift
		 *  an die im ControlCenter angegebene URL und gibt Debuginformationen darüber zurück
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=1)  aktiviert Testumgebung
		 * @param string $transactionId Transaktionsnummer einer PURCHASE- oder CAPTURE-Transaktion
		 * 
		 * @return string 
		 */
		public function transactionBackpayNotificationTest($accessKey, $testMode=1, $transactionId);

		/**
		 * Versendet eine Benachrichtigung über einen Refund
		 *  an die im ControlCenter angegebene URL und gibt Debuginformationen darüber zurück
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=1)  aktiviert Testumgebung
		 * @param string $transactionId Transaktionsnummer einer REFUND-Transaktion
		 * 
		 * @return string 
		 */
		public function transactionRefundNotificationTest($accessKey, $testMode=1, $transactionId);

		/**
		 * Versendet eine Benachrichtigung über eine NICHT transaktionsbezogene Gutschrift
		 *  an die im ControlCenter angegebene URL und gibt Debuginformationen darüber zurück
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=1)  aktiviert Testumgebung
		 * @param string $transactionId Transaktionsnummer einer CREDIT-Transaktion
		 * 
		 * @return string 
		 */
		public function transactionCreditNotificationTest($accessKey, $testMode=1, $transactionId);

		/**
		 * Gibt eine Liste von Bezahlvorgängen anhand von Parametern zurück
		 * 
		 *  Folgende frei kombinierbare Parameter stehen zur Verfügung:
		 *  - Kunde (customerId)
		 *  - Zeitraum von/bis (dtmFrom/dtmTo)
		 *  - SessionStatus (status)
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId (default=null)  eindeutige ID des Kunden oder [NULL] ohne Kundeneinschränkung
		 * @param datetime $dtmFrom (default=null)  Anfang der Zeitraumbegrenzung oder [NULL] um diese Zeitraumbegrenzung nicht zu definieren
		 * @param datetime $dtmTo (default=null)  Ende der Zeitraumbegrenzung oder [NULL] um diese Zeitraumbegrenzung nicht zu definieren
		 * @param SessionStatus $status (default=null)  Nur Sessions mit dem gewünschten Status in das Ergebnisliste aufnehmen oder [NULL] um Statusfilter zu deaktivieren
		 * 
		 * @return string[] 
		 */
		public function sessionList($accessKey, $testMode=0, $customerId=null, $dtmFrom=null, $dtmTo=null, $status=null);

		/**
		 * Errechnet ein Scoring für einen zuvor angelegten Kunden, den zugehörigen Kreditkartendaten sowie der IP-Adresse und ggf. Http-Request-Headern des Kunden.
		 *  Daher ist es für die Verwendung sinnvoll möglichst alle Daten zu erfassen um die Genauigkeit des Scorings zu erhöhen.
		 * 
		 *  Begrifflichkeiten:
		 *   E-Mail           E-Mail-Adressen aus Kundendaten
		 *   BillingCountry   Land des Kunden aus Adressendaten des Kunden [billCountry]
		 *   BillingLocation  Standort des Kunden aus Adressendaten des Kunden
		 *   BinCountry       Land der Kreditkartenbank (BIN-Code) [ccCountry]
		 *   IPCountry        Land durch IP ermittelt [ipCountry]
		 *   IPLocation       Standort ermittelt durch IP
		 * 
		 *  Wichtung  Wert
		 *     2.5    E-Mail (Anbieter von Temporären E-Mail-Adressen / Freemail Provider > höheres Risiko)
		 *     2.5    IPCountry != BillingCountry
		 *     2.0    BillingCountry != BinCountry
		 *     5.0    customerIp ist Anonymous-Proxy
		 *     2.5    customerIp ist bekannter Proxy
		 *    10.0    Entfernung IPLocation zu BillingLocation (höhere Entfernung = höheres Risiko)
		 *     5.0    E-Mail steht auf Badlist (Hochrisiko)
		 *     5.0    BillingLocation ist bekannter Maildrop (Postfach etc.)
		 *     5.0    IPCountry oder BillingCountry sind Hochrisikoländer - z.Z. sind das:
		 *              Egypt, Ghana, Indonesia, Lebanon, Macedonia, Morocco, Nigeria,
		 *              Pakistan, Romania, Serbia and Montenegro, Ukraine, Vietnam
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId eindeutige ID des Kunden
		 * @param string $customerIp IP des Benutzers
		 * @param map $customerRequestHeader (default=null)  Liste mit Http-Request-Headern des Kunden - ausgewertet werden: User-Agent, Accept-Language sowie die Proxy-Header X-Forwarded-For/Client-Ip
		 * 
		 * @return struct 
		 * @result string $billCountry (default=null)  Land [ISO 3166-1-alpha-2] aus den Adressdaten des Kundensatzes bspw. DE, AT, CH
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result float $scoreValue (default=null)  Bewertung mit Werten zwischen 0 und 100, wobei 0 für geringes und 100 für hohes Risiko steht
		 * @result string $scoreExplanation (default=null)  Erläuterung zu der Bewertung in Textform [Englisch]
		 */
		public function customerScoring($accessKey, $testMode=0, $customerId, $customerIp, $customerRequestHeader=null);

		/**
		 * Transaktion zur Buchung einer Gesamt- oder Teilrückzahlung einer den Kunden belastende Buchung.
		 * 
		 *  Sie erstatten den gebuchten Betrag:
		 *  - In voller Höhe (Gesamtbetrag) - der Parameter `amount` entfällt ([NULL])
		 *  - Oder nur einen Teilbetrag - der Parameter `amount` enthält den gewünschten Teilbetrag
		 * 
		 *  Zur Durchführung der Transaktion ist es notwendig das Sie mittels `sessionCreate` einen Bezahlvorgang erstellt,
		 *  sowie unter Verwendung von `transactionPurchase` oder `transactionCapture` eine den Kunden belastende Buchung durchgeführt haben.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * @param string $transactionId Transaktionsnummer der Transaktion die zurückgebucht werden soll
		 * @param integer $amount (default=null)  zurückzubuchender Betrag, falls abweichend von Orginaltransaktion
		 * @param string $currency (default=null)  Währung, falls abweichend von Originaltransaktion [nur relevant wenn `amount` abweichend]
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * 
		 * @return struct 
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 */
		public function transactionRefund($accessKey, $testMode=0, $sessionId, $transactionId, $amount=null, $currency=null, $fraudDetection=1);

		/**
		 * Transaktion zur Buchung einer nicht transaktionsbezogenen Gutschrift.
		 * 
		 *  Dies ermöglicht es Ihnen beliebige Beträge auf das Kreditkartenkonto einen Kunden zu übertragen.
		 * 
		 *  Zur Durchführung der Transaktion ist es notwendig das Sie mittels `sessionCreate` einen Bezahlvorgang erstellt haben,
		 *  sowie dem Kunden, unter Zuhilfenahme von `creditcardDataSet`, Kreditkartendaten zugewiesen haben.
		 * 
		 *  HINWEIS:
		 *  Diese Funktionalität bedarf manueller Freischaltung, und wird vom buchbaren Volumen beschränkt
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * @param integer $amount (default=null)  zurückzubuchender Betrag, falls abweichend von Orginaltransaktion
		 * @param string $currency (default=null)  Währung, falls abweichend von Originaltransaktion [nur relevant wenn `amount` abweichend]
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * 
		 * @return struct 
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $ccCountry (default=null)  Land [ISO 3166-1-alpha-2] der Kreditkarte wenn verfügbar, bspw. DE, AT, CH
		 * @result string $ipCountry (default=null)  Land [ISO 3166-1-alpha-2] der IP wenn verfügbar, bspw. GB, US
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 */
		public function transactionCredit($accessKey, $testMode=0, $sessionId, $amount=null, $currency=null, $fraudDetection=1);

		/**
		 * Zuordnung einer `dataStorageId` von einem Bestandskunden Ihres Systems zu einem Kunden (customerId) bei micropayment
		 * 
		 *  Wichtig:
		 *  Diese Funktion ist nur bei einem Wechsel von einem anderen DataStorage-Anbieter relevant und setzt den
		 *  Import der zugehörigen Kreditkartendaten vorraus separat beantragt und durchgeführt werden muss.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * @param string $dataStorageId Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * 
		 * @return boolean 
		 */
		public function dataStorageIdImport($accessKey, $testMode=0, $customerId, $dataStorageId);

		/**
		 * Gibt die mit einem Kunden verknüpften Kreditkartendaten zurück.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * 
		 * @return struct 
		 * @result string $type (default=null)  Kartentyp
		 * @result string $number (default=null)  partielle Kreditkartennummer [letzten 4 Stellen]
		 * @result integer $expiryYear (default=null)  Gültigkeits Jahr
		 * @result integer $expiryMonth (default=null) 	Gültigkeits Monat
		 * @result boolean $cvc2Required Bei der nächsten Buchung ist der CVC2-Code erforderlich
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 */
		public function creditcardDataGet($accessKey, $testMode=0, $customerId);

		/**
		 * Transaktion zur gebührenfreier Stornierung einer Buchung vor Kassenschnitt oder Freigabe von Vorautorisierungen.
		 * 
		 *  Die gebührenfreie Stornierung einer mit `transactionPurchase` durchgef?hrten Buchung muss am gleichen Tag vor Kassenschnitt um 24 Uhr erfolgen.
		 *  Die Freigabe einer mittels `transactionAuthorization` erstellten Vorautorisierungen hingegen kann jederzeit durchgeführt werden.
		 * 
		 *  Zur Durchführung der Transaktion ist es notwendig das Sie mittels `sessionCreate` einen Bezahlvorgang erstellt,
		 *  sowie unter Verwendung von `transactionAuthorization` eine Vorautorisierung oder mit `transactionPurchase` eine Buchung durchgeführt haben.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId ID des Vorgangs
		 * @param string $transactionId Transaktionsnummer der Transaktion die zurückgebucht werden soll
		 * @param integer $fraudDetection (default=1)  de/aktiviert FraudDetection
		 * 
		 * @return struct 
		 * @result SessionStatus $sessionStatus Status der gesamten Session
		 * @result TransactionStatus $transactionStatus Status der ausgelösten Transaktion
		 * @result string $transactionId Transaktionsnummer
		 * @result datetime $transactionCreated Zeitpunkt der Transaktion
		 * @result string $transactionAuth AuthCode
		 * @result string $dataStorageId (default=null)  Wenn DataStorage-Feature aktiv, ID der Kreditkartendaten im DataStorage-Service
		 * @result string $transactionResultCode Transaktionergebnis-Code vom Terminal
		 * @result string $transactionResultMessage Transaktionergebnis-Nachricht vom Terminal
		 */
		public function transactionReversal($accessKey, $testMode=0, $sessionId, $transactionId, $fraudDetection=1);

		/**
		 * Gibt eine Liste der, vom System durch Umrechnung, unterstützten Währungen zurück.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * 
		 * @return string[] 
		 */
		public function currenciesGetSupported($accessKey, $testMode=0);

		/**
		 * Gibt eine Liste der, von Ihrem Account, nativ unterstützten Währungen, optional für einen bestimmten Kartentyp, zurück.
		 * 
		 *  Hinweis:
		 *  Die native Unterstützung weiterer Währungen erhalten Sie durch die Beantragung zusätzlicher VU-Nummern.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param CardType $cardtype (default=null)  Typ der Kreditkarte
		 * 
		 * @return string[] 
		 */
		public function currenciesGetNativeSupported($accessKey, $testMode=0, $cardtype=null);

		/**
		 * Überprüfung von Kreditkarten-Daten (Nummer und Ablaufdatum) auf Gültigkeit
		 * 
		 *  - syntaktischen Überprüfung
		 *  - Luhn-Check
		 *  - Überschreitung des Ablaufdatums
		 * 
		 *  Hinweis: Es findet keine Online-Überprüfung der Daten statt!
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $number Kreditkartennummer
		 * @param integer $expiryYear Gültigkeits Jahr
		 * @param integer $expiryMonth Gültigkeits Monat
		 * 
		 * @return boolean Kreditkarten-Daten sind syntaktisch gültig
		 */
		public function creditcardCheckValidity($accessKey, $testMode=0, $number, $expiryYear, $expiryMonth);

		/**
		 * Löscht alle im Testmodus übertragenen Daten
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=1)  aktiviert Testumgebung
		 * 
		 * @return boolean 
		 */
		public function resetTest($accessKey, $testMode=1);

		/**
		 * Versendet eine Benachrichtigung über ein Chargeback an die im
		 *  ControlCenter angegebene URL und gibt Debuginformationen darüber zurück
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=1)  aktiviert Testumgebung
		 * @param string $transactionId Transaktionsnummer einer PURCHASE- oder CAPTURE-Transaktion
		 * 
		 * @return string 
		 */
		public function transactionChargebackNotificationTest($accessKey, $testMode=1, $transactionId);

		/**
		 * Verknüpft Adressdaten mit einem bestehenden Kunden
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)   aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * @param string $address Strasse und Hausnummer
		 * @param string $zipcode Postleitzahl
		 * @param string $town Ort
		 * @param string $country Land [ISO 3166-1-alpha-2] bspw. DE, AT, CH
		 * 
		 * @return boolean 
		 */
		public function addressSet($accessKey, $testMode=0, $customerId, $address, $zipcode, $town, $country);

		/**
		 * Gibt die mit einem Kunden verknüpften Adressdaten zurück.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)   aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * 
		 * @return array 
		 * @result string $address Strasse und Hausnummer
		 * @result string $zipcode Postleitzahl
		 * @result string $town Ort
		 * @result string $country Land [ISO 3166-1-alpha-2] bspw. DE, AT, CH
		 */
		public function addressGet($accessKey, $testMode=0, $customerId);

		/**
		 * Ändert Daten eines bestehenden Kunden
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId eindeutige ID des Kunden
		 * @param map $freeParams (default=null)  Liste mit freien Parametern: [NULL] - Parameterliste bleibt unverändert | leeres HashMap - löscht Parameterliste | gefülltes HashMap erweitert/überschreibt bestehende Parameterliste
		 * @param string $firstname (default=null)  Vorname des Kunden: [NULL] - aktueller Wert bleibt erhalten | gültiger Wert z.B 'Max'
		 * @param string $surname (default=null)  Nachname des Kunde: [NULL] - aktueller Wert bleibt erhalten | gültiger Wert z.B 'Mustermann'
		 * @param string $email (default=null)  E-Mail-Adresse des Kunden: [NULL] - aktueller Wert bleibt erhalten | gültiger Wert z.B. 'max@mustermann.de' ersetzt den aktuellen Wert
		 * @param string $culture (default=null)  Sprache & Land des Kunden: [NULL] - aktueller Wert bleibt erhalten | gültige Wert z.B. 'de-DE' ersetzt den aktuellen Wert
		 * 
		 * @return boolean 
		 */
		public function customerSet($accessKey, $testMode=0, $customerId, $freeParams=null, $firstname=null, $surname=null, $email=null, $culture=null);

		/**
		 * Gibt die Daten eines bestehenden Kunden zurück.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * 
		 * @return struct 
		 * @result map $freeParams (default=null)  Liste mit allen freien Parametern
		 * @result string $firstname (default=null)  Vorname des Kunden
		 * @result string $surname (default=null)  Nachname des Kunden
		 * @result string $email (default=null)  E-Mail-Adresse des Kunden
		 * @result string $culture (default=null)  Sprache & Land des Kunden
		 */
		public function customerGet($accessKey, $testMode=0, $customerId);

		/**
		 * Erzeugt einen neuen Bezahlvorgang für einen Kunden
		 * 
		 *  Zur Durchführung wird zwingender Weise ein Kunde (customerId) benötigt, den Sie mittels `customerCreate` anlegen können.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $customerId ID des Kunden
		 * @param string $sessionId (default=null)  eigene eindeutige ID des Vorgangs, wird anderenfalls erzeugt [max. 40 Zeichen]
		 * @param string $project das Projektkürzel für den Vorgang
		 * @param string $projectCampaign (default=null)  ein Kampagnenkürzel des Projektbetreibers
		 * @param string $account (default=null)  Account des beteiligten Webmasters sonst eigener - setzt eine Aktivierung der Webmasterfähigkeit des Projekts vorraus - Hinweis: Webmasterfähigkeit steht momentan nicht zur Verfügung
		 * @param string $webmasterCampaign (default=null)  ein Kampagnenkürzel des Webmasters
		 * @param integer $amount (default=null)  abzurechnender Betrag, wird kein Betrag übergeben, wird der Betrag aus der Konfiguration verwendet
		 * @param string $currency (default='EUR')  Währung
		 * @param string $title (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung in Falle einer auftretenden Benachrichtigung wird dieser Wert als Produktidentifizierung mit geschickt, wird kein Wert übergeben, wird Der aus der Konfiguration verwendet
		 * @param string $paytext (default=null)  Bezeichnung der zu kaufenden Sache - Verwendung beim Mailversand, sollten Sie Diesen wünschen
		 * @param string $ip IPv4 oder IPv6 des Benutzers
		 * @param map $freeParams (default=null)  Liste mit freien Parametern, die dem Vorgang zugeordnet werden
		 * @param boolean $sendMail (default=true) 
		 * 
		 * @return struct 
		 * @result string $sessionId eigene oder erzeugte eindeutige ID des Vorgangs
		 * @result SessionStatus $status Vorgangsstatus "INIT"
		 * @result datetime $expire Ablaufzeit des Vorgangs
		 */
		public function sessionCreate($accessKey, $testMode=0, $customerId, $sessionId=null, $project, $projectCampaign=null, $account=null, $webmasterCampaign=null, $amount=null, $currency='EUR', $title=null, $paytext=null, $ip, $freeParams=null, $sendMail=true);

		/**
		 * Gibt Informationen über einen Bezahlvorgang, inklusive einer Liste von verknüpften Transaktionen, zurück.
		 *
		 * @param string $accessKey AccessKey aus dem Controlcenter
		 * @param integer $testMode (default=0)  aktiviert Testumgebung
		 * @param string $sessionId eindeutige ID des Vorgangs
		 * 
		 * @return struct 
		 * @result string $customerId ID des Kunden
		 * @result string $project das Projektkürzel für den Vorgang
		 * @result string $projectCampaign ein Kampagnenkürzel des Projektbetreibers
		 * @result string $account Account des beteiligten Webmasters sonst eigener
		 * @result string $webmasterCampaign ein Kampagnenkürzel des Webmasters
		 * @result integer $amount abzurechnender Betrag, wird kein Betrag übergeben, wird der Betrag aus der Konfiguration verwendet
		 * @result string $currency Währungseinheit
		 * @result string $title Bezeichnung der zu kaufenden Sache
		 * @result string $ip IPv4 oder IPv6 des Benutzers
		 * @result map $freeParams (default=null)  Liste mit freien Parametern, die dem Vorgang zugeordnet werden
		 * @result SessionStatus $status 
		 * @result datetime $expire (default=null)  Verfallsdatum der Session, nur wenn $status INIT oder EXPIRED
		 * @result MailStatus $mail Status des Mailversands
		 * @result string[] $transactionIds (default=null)  Liste von TransaktionsIds die mit dieser Session verknüpft sind
		 */
		public function sessionGet($accessKey, $testMode=0, $sessionId);

	}

?>